#include <iostream>
#include "libharkio.h"

using namespace std;
using namespace harkio;

// テスト対象のコメントを外してください
#define M2PG_TEST
//#define GTF_TEST
//#define SM_TEST

void harkTest() {

	try {
		/*******************************************************************
		 / HARK RIFFオブジェクト生成
		 *******************************************************************/
		// HARK　RIFF　生成
		HARK hark_write;
		Data *hgtf;

		/*******************************************************************
		 / HGTF Dataオブジェクト生成
		 *******************************************************************/
		hgtf = new HGTF();
		hark_write.addData(hgtf); // hark_writeにdataの追加

		/*******************************************************************
		 / 各種、FMTチャンクの作成
		 *******************************************************************/
		// 書式 FMT(FTM::VNAME version, FMT::FORMAT_ID format_id, int nMic, int nFFT, int nIR, int nFS)
#ifdef M2PG_TEST
		FMT *fmt = new FMT(FMT::V1_0, FMT::M2PG, 0x2, 0x4, 0x8, 0xF);
#endif
#ifdef 	GTF_TEST
		FMT *fmt = new FMT(FMT::V1_0, FMT::GTF, 0x2, 0x4, 0x8, 0xF);
#endif
#ifdef SM_TEST
		FMT *fmt = new FMT(FMT::V1_0, FMT::SM, 0x2, 0x4, 0x8, 0xF);
#endif
		hgtf->addChunk(fmt); // hgtfへFMTチャンクの追加

		// FMT Chunkへマイク座標コンテナを生成して、サンプルデータをセット
		FMT::MIC_LIST_TYPE *mic = fmt->newMic(); // マイク座標コンテナ生成
		(*mic)[0].x = 0; // 以下、サンプルデータ
		(*mic)[0].y = 0xF;
		(*mic)[0].z = 0xFF;

#ifdef M2PG_TEST// M2PGの場合
		/*******************************************************************
		 / MUSIC 用　M2PG チャンクの作成
		 *******************************************************************/
		// 書式M2PG(M2PG::VNAME version, FMT *fmt, int nd, int nr);
		M2PG *m2pg = new M2PG(M2PG::V1_0, fmt, 0x3, 0x1);
		hgtf->addChunk(m2pg);

		// Chunk から音源位置座標コンテナを取得して、テストデータをセット
		M2PG::SRC_LIST_TYPE *src = m2pg->getSrc();
		(*src)[0][0].x = 0;
		(*src)[0][0].y = 0xF;
		(*src)[0][0].z = 0xFF;

		// Chunk から伝達関数コンテナを取得して、テストデータをセット
		M2PG::TF_LIST_TYPE *tf = m2pg->getTF();
		(*tf)[0][0][0][0].real() = 0x0;
		(*tf)[0][0][0][0].imag() = 0xF;
		// ここまでで、コンテナへデータセット完了
#endif

#ifdef GTF_TEST
		/*******************************************************************
		 / GSS 用　GTF チャンクの作成
		 *******************************************************************/
		// 書式GTF(GTF::VNAME version, FMT *fmt, int n_src, int n_fi);
		GTF *gtf = new GTF(GTF::V1_0, fmt, 0x4, 0x2);
		hgtf->addChunk(gtf);

		// Chunk から音源位置座標コンテナを取得して、テストデータをセット
		GTF::SRC_LIST_TYPE *src = gtf->getSrc();
		(*src)[0].x = 1;
		(*src)[0].y = 0x11;
		(*src)[0].z = 0x101;

		// Chunk から伝達関数コンテナを取得して、テストデータをセット
		GTF::TF_LIST_TYPE *tf = gtf->getTF();
		(*tf)[0][0][0].real() = 0x1;
		(*tf)[0][0][0].imag() = 0x11;

		// 伝達関数数の周波数インデックスからインデックスへのマップ、コンテナを取得して、テストデータをセット
		vector<int> *fi = gtf->getFI();
		(*fi)[0] = 0xF;
#endif
#ifdef SM_TEST
		/*******************************************************************
		 / GSS 用　SM チャンクの作成
		 *******************************************************************/
		// 書式 SM(SM::VNAME version, FMT *fmt, int ns, int nf);
		SM *sm = new SM(SM::V1_0, fmt, 0x5, 0x3);
		hgtf->addChunk(sm);

		// Chunk から音源位置座標コンテナを取得して、テストデータをセット
		SM::SRC_LIST_TYPE *src = sm->getSrc();
		(*src)[0].x = 1;
		(*src)[0].y = 0x11;
		(*src)[0].z = 0x101;

		// Chunk から分離行列コンテナを取得して、テストデータをセット
		SM::SM_LIST_TYPE *sm_data = sm->getSM();
		(*sm_data)[0][0][0].real() = 0x1;
		(*sm_data)[0][0][0].imag() = 0x11;

		// 分離行列の周波数インデックスからインデックスへのマップ、コンテナを取得して、テストデータをセット
		vector<int> *fi = sm->getFI();
		(*fi)[0] = 0xF;
#endif
		/*******************************************************************
		 / 　共通　SPEC チャンクの作成
		 *******************************************************************/
		SPEC *spec = new SPEC(SPEC::V1_0);
		hgtf->addChunk(spec);
		// SPEC データ作成
		spec->setDate("2009/1/1");
		spec->setLocation("Location Test Data");
		spec->setRobot("Robot Test Data");
		spec->setArrayType("Array Type Test Data");
		spec->setCreator("Creator Test Data");
		spec->setMemo("Memo Test Data");

		// hark_write全体を、引数のファイルへの書き込みを行う
		hark_write.writeFile("./test.dat");

		/*******************************************************************
		 / ここから、作成した各種ファイルの読み込み
		 *******************************************************************/
		// ファイルオープンの前に、識別子のチェックを行い、ファイルの種類を判定する
		if (HARK::checkIdentifier("./test.dat") != HARKIO_TRUE) {
			cout << harkio::Error::getInstance().getMsg() << endl;
			return;
		}
		// 読み込み用のhark_readを用意します
		HARK hark_read;

		// 正式にファイルを読み込む
		hark_read.readFile("./test.dat");

		// 以下、読み込んだファイルの中身の参照を行います
		{
			Chunk::VERSION_CHECK version_check;
			/*******************************************************************
			 / FMT チャンクの内容
			 *******************************************************************/
			cout << " --- FMT Chunk --- " << endl;
			FMT *fmt = hark_read.getData()->getChunk<FMT> ();

			// バージョンチェック
			version_check = fmt->checkVersion(FMT::V1_0); // 要求バージョン
			if (version_check == Chunk::MISMATCH) {
				cout << harkio::Error::getInstance().getMsg() << endl;
				return;
			} else if (version_check == Chunk::SAFE_MATCH) { // チェックしなくても良い
				cout << harkio::Error::getInstance().getMsg() << endl;
			} else {
				cout << "VERSION MATCH OK!" << endl;
			}

			cout << " Size     =" << fmt->getSize() << endl;
			cout << " Version = " << fmt->getVersionStr() << endl; // 他のバリエーションもあります
			cout << " FormatId =" << fmt->getFormatId() << endl;
			cout << " NMic     =" << fmt->getNMic() << endl;
			cout << " NFFT     =" << fmt->getNFFT() << endl;
			cout << " NIR      =" << fmt->getNIR() << endl;
			cout << " NFS      =" << fmt->getNFS() << endl;

			FMT::MIC_LIST_TYPE *mic = fmt->getMic();
			if (mic->size()) {
				cout << "sample mic[0]: " << endl;
				cout << " x:" << (*mic)[0].x << endl;
				cout << " y:" << (*mic)[0].y << endl;
				cout << " z:" << (*mic)[0].z << endl;
			}
#ifdef M2PG_TEST
			/*******************************************************************
			 / M2PG チャンクの内容
			 *******************************************************************/
			cout << " --- M2PG Chunk --- " << endl;
			M2PG *m2pg = hark_read.getData()->getChunk<M2PG> ();

			// バージョンチェック
			version_check = m2pg->checkVersion(M2PG::V1_0); // 要求バージョン
			if (version_check == Chunk::MISMATCH) {
				cout << harkio::Error::getInstance().getMsg() << endl;
				return;
			} else if (version_check == Chunk::SAFE_MATCH) { // チェックしなくても良い
				cout << harkio::Error::getInstance().getMsg() << endl;
			} else {
				cout << "VERSION MATCH OK!" << endl;
			}

			cout << " Size =" << m2pg->getSize() << endl;
			cout << " Version =" << m2pg->getVersionStr() << endl; // 他のバリエーションもあります
			cout << " ND   =" << m2pg->getND() << endl;
			cout << " NR   =" << m2pg->getNR() << endl;

			// 音源位置座標(1 sample)
			M2PG::SRC_LIST_TYPE *src = m2pg->getSrc();
			//			if (src->size()) {
			for (int i = 0; i < src->size(); i++) {
				cout << "sample src[0][0] :" << endl;
				cout << " x:" << (*src)[i][0].x << endl;
				cout << " y:" << (*src)[i][0].y << endl;
				cout << " z:" << (*src)[i][0].z << endl; // src[nd][nr]
			}
			// 伝達関数(1 sample)
			M2PG::TF_LIST_TYPE *tf = m2pg->getTF();
			if (tf->size()) {
				cout << "sample tf[0] : " << (*tf)[0][0][0][0] << endl; // tf[nd][nr][nmic][nfft]
			}
#endif

#ifdef GTF_TEST
			/*******************************************************************
			 / GTF チャンクの内容
			 *******************************************************************/
			cout << " --- GTF Chunk --- " << endl;
			GTF *gtf = hark_read.getData()->getChunk<GTF>();

			// バージョンチェック
			version_check = gtf->checkVersion(GTF::V1_0); // 要求バージョン
			if (version_check == Chunk::MISMATCH) {
				cout << harkio::Error::getInstance().getMsg() << endl;
				return;
			} else if (version_check == Chunk::SAFE_MATCH) { // チェックしなくても良い
				cout << harkio::Error::getInstance().getMsg() << endl;
			} else {
				cout << "VERSION MATCH OK!" << endl;
			}

			cout << " Size =" << gtf->getSize() << endl;
			cout << " Version =" << gtf->getVersionStr() << endl; // 他のバリエーションもあります
			cout << " NSrc =" << gtf->getNSrc() << endl;
			cout << " NFI  =" << gtf->getNFI() << endl;

			// 音源位置座標(1 sample)
			GTF::SRC_LIST_TYPE *src = gtf->getSrc();
			if (src->size()) {
				cout << "sample src[0][0] :"<< endl;
				cout << " x:" << (*src)[0].x << endl;
				cout << " y:" << (*src)[0].y << endl;
				cout << " z:" << (*src)[0].z << endl; // src[fs]
			}
			// 伝達関数(1 sample)
			GTF::TF_LIST_TYPE *tf = gtf->getTF();
			if (tf->size()) {
				cout << "sample tf[0] : " << (*tf)[0][0][0] << endl; // tf[ns][nmic][nf]
			}
			// 伝達関数数の周波数インデックスからインデックスへのマップ(1 sample)
			vector<int> *fi = gtf->getFI();
			if (fi->size()) {
				cout << "sample fi[0] : " << (*fi)[0] << endl; // fi[nf]
			}
#endif

#ifdef SM_TEST
			/*******************************************************************
			 / SM チャンクの内容
			 *******************************************************************/
			cout << " --- SM Chunk --- " << endl;
			SM *sm = hark_read.getData()->getChunk<SM>();

			// バージョンチェック
			version_check = sm->checkVersion(SM::V1_0); // 要求バージョン
			if (version_check == Chunk::MISMATCH) {
				cout << harkio::Error::getInstance().getMsg() << endl;
				return;
			} else if (version_check == Chunk::SAFE_MATCH) { // チェックしなくても良い
				cout << harkio::Error::getInstance().getMsg() << endl;
			} else {
				cout << "VERSION MATCH OK!" << endl;
			}

			cout << " Size =" << sm->getSize() << endl;
			cout << " Version =" << sm->getVersionStr() << endl; // 他のバリエーションもあります
			cout << " NS   =" << sm->getNSrc() << endl;
			cout << " NF   =" << sm->getNFI() << endl;

			// 音源位置座標(1 sample)
			SM::SRC_LIST_TYPE *src = sm->getSrc();
			if (src->size()) {
				cout << "sample src[0][0] :"<< endl;
				cout << " x:" << (*src)[0].x << endl;
				cout << " y:" << (*src)[0].y << endl;
				cout << " z:" << (*src)[0].z << endl; // src[fs]
			}
			// 伝達関数数の周波数インデックスからインデックスへのマップ(1 sample)
			vector<int> *fi = sm->getFI();
			if (fi->size()) {
				cout << "sample fi[0] : " << (*fi)[0] << endl; // fi[nf]
			}
			// 分離行列(1 sample)
			SM::SM_LIST_TYPE *sm_data = sm->getSM();
			if (sm_data->size()) {
				cout << "sample sm[0] : " << (*sm_data)[0][0][0] << endl; // sm_data[ns][nmic][nf]
			}
#endif
			/*******************************************************************
			 / SPEC チャンクの内容
			 *******************************************************************/
			cout << " --- SPEC Chunk --- " << endl;
			SPEC *spec = hark_read.getData()->getChunk<SPEC> ();

			// バージョンチェック
			version_check = spec->checkVersion(SPEC::V1_0); // 要求バージョン
			if (version_check == Chunk::MISMATCH) {
				cout << harkio::Error::getInstance().getMsg() << endl;
				return;
			} else if (version_check == Chunk::SAFE_MATCH) { // チェックしなくても良い
				cout << harkio::Error::getInstance().getMsg() << endl;
			} else {
				cout << "VERSION MATCH OK!" << endl;
			}
			cout << spec->getDate() << endl;
			cout << spec->getLocation() << endl;
			cout << spec->getRobot() << endl;
			cout << spec->getArrayType() << endl;
			cout << spec->getCreator() << endl;
			cout << spec->getMemo() << endl;
		}
		// テスト目的で、もう一度書き戻して、test.dat,test.bak のバイナリが一致するか確認してみる
		hark_read.writeFile("./test.bak");
		// バイナリサイズを比較してみる
		cout << "Binary size is " << hark_read.getClassSize() << " Byte" << endl;

	} catch (harkio::Error& err) {
		cout << "Error Message = " << err.getMsg() << endl;
		throw;
	}
	/*
	 } catch(std::exception& expt) {
	 harkio::Error* err = dynamic_cast<harkio::Error*>(&expt);
	 if (err) {
	 cout << "Error Message = " << err->getMsg() << endl;
	 }
	 throw;
	 }
	 */
}
